@@ -31,13 +31,19 @@
 #include <unistd.h>
 #include <sndfile.h>
 #include <pthread.h>
+#ifdef __APPLE__
+#include <mach/mach.h>
+#else
 #include <semaphore.h>
+#endif
 #include <math.h>
 #include <stdarg.h>
 
 #include <termios.h>
 
+#ifndef __APPLE__
 #include <sys/sysinfo.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -60,6 +66,8 @@ int  init_osc(int osc_port);
 void shutdown_osc(void);
 #endif
 
+#include "atomic.h"
+
 #include "vringbuffer.h"
 
 
@@ -109,7 +117,7 @@ static int vu_dB=true;
 static float vu_bias=1.0f;
 static int leading_zeros=1;
 static char *filename_prefix="jack_capture_";
-static int64_t num_frames_recorded=0;
+static DEFINE_ATOMIC(int64_t, num_frames_recorded)=0;
 static int64_t num_frames_to_record=0;
 static bool fixed_duration=false;
 static bool no_stdin=false;
@@ -149,7 +157,7 @@ static int64_t num_frames_written_to_disk = 0;
 static bool program_ended_with_return = false;
 
 /* JACK data */
-static jack_port_t **ports;
+static DEFINE_ATOMIC(jack_port_t**, g_ports) = NULL;
 static jack_port_t **ports_meterbridge=NULL;
 typedef jack_default_audio_sample_t sample_t;
 static float jack_samplerate;
@@ -179,8 +187,8 @@ static float *vu_peakvals=NULL;
 static void print_message(const char *fmt, ...);
 
 /* Synchronization between jack process thread and disk thread. */
-static volatile int is_initialized=0; // This $@#$@#$ variable is needed because jack ports must be initialized _after_ (???) the client is activated. (stupid jack)
-static volatile int is_running=1; // Mostly used to stop recording as early as possible.
+static DEFINE_ATOMIC(bool, is_initialized) = false; // This $@#$@#$ variable is needed because jack ports must be initialized _after_ (???) the client is activated. (stupid jack)
+static DEFINE_ATOMIC(bool, is_running) = true; // Mostly used to stop recording as early as possible.
 
 
 /* Buffer */
@@ -204,8 +212,11 @@ static pthread_t connect_thread={0} ;
 
 
 // das stop semaphore
+#ifdef __APPLE__
+static semaphore_t stop_sem;
+#else
 static sem_t stop_sem;
-
+#endif
 
 
 
@@ -253,7 +264,7 @@ static bool set_high_priority(void){
     prio++;
   
   if(prio==0 && shown_warning==false){
-    print_message("Warning. Could not set higher priority for a SCHED_FIFO process using setpriority().\n");
+    print_message("Warning. Could not set higher priority for a SCHED_OTHER process using setpriority().\n");
     shown_warning=true; }
 
   if(prio < 0)
@@ -334,6 +345,9 @@ static float buffers_to_seconds(int buffers){
 
 
 static int autoincrease_callback(vringbuffer_t *vrb, bool first_call, int reading_size, int writing_size){
+  (void)vrb;
+  (void)reading_size;
+
   if(use_jack_freewheel)
     return 0;
 
@@ -483,6 +497,8 @@ static void portnames_add(char *name){
 }
 
 static const char **portnames_get_connections(int ch, bool *using_calloc){
+  *using_calloc = true; // silence compiler warning. (fix: logic of program is too complicated)
+  
   if(ch>=num_cportnames)
     return NULL;
   else{      
@@ -660,7 +676,7 @@ static void print_console(bool move_cursor_to_top_doit,bool force_update){
         if(vu_peakvals[ch]>=1.0f)
           printf("%c[31m",0x1b); // Peaking, show red color
 
-        printf(vol);
+        printf("%s", vol);
 
         for(;i<vu_len;i++)
           vol[4+i] = vu_not_recording[i];
@@ -668,7 +684,7 @@ static void print_console(bool move_cursor_to_top_doit,bool force_update){
         printf("%c[33m",0x1b); // Yellow color
 
         vol[i+4]='\0';
-        printf(vol+4+pos);
+        printf("%s", vol+4+pos);
 
         printf("%c[36m",0x1b); // back to cyan
         printf("|\n");
@@ -702,7 +718,7 @@ static void print_console(bool move_cursor_to_top_doit,bool force_update){
     int   num_buffers = (vringbuffer_reading_size(vringbuffer)+ vringbuffer_writing_size(vringbuffer));
     float buflen      = buffers_to_seconds(num_buffers);
     float bufleft     = buffers_to_seconds(num_bufleft);
-    int   recorded_seconds = (int)frames_to_seconds(num_frames_recorded);
+    int   recorded_seconds = (int)frames_to_seconds(ATOMIC_GET(num_frames_recorded));
     if(timemachine_mode==true)
       recorded_seconds = (int)frames_to_seconds(num_frames_written_to_disk);
     int   recorded_minutes = recorded_seconds/60;
@@ -758,6 +774,8 @@ static int init_meterbridge_ports();
 static bool is_helper_running=true;
 
 static void *helper_thread_func(void *arg){
+  (void)arg;
+
   helper_thread_running=1;
 
   if(use_vu||show_bufferusage)
@@ -890,6 +908,7 @@ static void stop_helper_thread(void){
 #ifdef _WIN32
 #define vsnprintf _vsnprintf
 #endif
+#ifndef __APPLE__
 int asprintf(char **buffer, char *fmt, ...) {
     /* Guess we need no more than 200 chars of space. */
     int size = 200;
@@ -927,6 +946,7 @@ int asprintf(char **buffer, char *fmt, ...) {
     return size;
 }
 #endif
+#endif
 
 #define ARGS_ADD_ARGV(FMT,ARG) \
   argv=(char**) realloc((void*)argv, (argc+2)*sizeof(char*)); \
@@ -942,6 +962,7 @@ int asprintf(char **buffer, char *fmt, ...) {
 }
 
 static void wait_child(int sig){
+  (void)sig;
   wait(NULL);
 }
 
@@ -1050,13 +1071,17 @@ static int64_t overruns=0;
 
 #if HAVE_LIBLO
 bool queued_file_rotate=false;
+#ifdef __APPLE__
+void osc_stop() { semaphore_signal(stop_sem); }
+#else
 void osc_stop() { sem_post(&stop_sem); }
+#endif
 void osc_tm_start() { timemachine_recording=true; }
 void osc_tm_stop() { program_ended_with_return=true; osc_stop(); }
 #endif
 
 static struct timespec rtime;
-static int store_sync = 0;
+static DEFINE_ATOMIC(int, g_store_sync) = 0;
 static int ssync_offset = 0;
 static jack_nframes_t j_latency = 0;
 
@@ -1135,7 +1160,7 @@ static int open_soundfile(void){
   if(filename==NULL){
     if (rotateframe>0){ // always use .NUM. with rotation
       filename=my_calloc(1,strlen(base_filename)+500);
-      sprintf(filename,"%s.%0*d.wav",base_filename,leading_zeros+1,0);
+      sprintf(filename,"%s.%0*d.%s",base_filename,leading_zeros+1,0,soundfile_format);
     }else{
       filename=strdup(base_filename);
     }
@@ -1255,7 +1280,7 @@ static void close_soundfile(void){
 }
 
 static int rotate_file(size_t frames, int reset_totals){
-	store_sync=0;
+  ATOMIC_SET(g_store_sync, 0);
 	// Explanation: new file will already contain the CURRENT buffer!
 	// but sync-timeframe will only be saved on the start of next jack cycle.
 	//
@@ -1266,7 +1291,7 @@ static int rotate_file(size_t frames, int reset_totals){
 
   char *filename_new;
   filename_new=my_calloc(1,strlen(base_filename)+500);
-  sprintf(filename_new,"%s.%0*d.wav",base_filename,leading_zeros+1,num_files);
+  sprintf(filename_new,"%s.%0*d.%s",base_filename,leading_zeros+1,num_files,soundfile_format);
   print_message("Closing %s, and continue writing to %s.\n",filename,filename_new);
   num_files++;
 
@@ -1414,7 +1439,7 @@ static int disk_write_overruns(int num_overruns){
                   "    Try a bigger buffer than -B %f\n%s",
                   num_overruns,num_overruns==1 ? "" : "s",
                   min_buffer_time,
-                  is_running != 0 ? "Continue recording...\n" : ""
+                  ATOMIC_GET(is_running) ? "Continue recording...\n" : ""
                   );
 
   overruns+=num_overruns;
@@ -1479,8 +1504,9 @@ static enum vringbuffer_receiver_callback_return_t disk_callback(vringbuffer_t *
 
   disk_thread_control_priority();
 
-  if (store_sync==1) {
-    store_sync=2;
+  
+  if (ATOMIC_COMPARE_AND_SET_INT(g_store_sync, 1, 2)) {
+    
     hook_rec_timimg(filename, rtime, j_latency);
 
     if (create_tme_file) { /* write .tme info-file */
@@ -1564,15 +1590,15 @@ static void process_fill_buffer(sample_t *in[],buffer_t *buffer,int i,int end){
         sample_t val=in[ch][i];
         data[pos++]=val;
         val=fabsf(val);
-        if(val>vu_vals[ch])
-          vu_vals[ch]=val;
+        if(val>safe_float_read(&vu_vals[ch]))
+          safe_float_write(&vu_vals[ch], val);
       }
     }
   }else{
     int start_i = i;
     for(ch=0;ch<num_channels;ch++){
       sample_t *curr_in=in[ch];
-      float max_vu=vu_vals[ch];
+      float max_vu=safe_float_read(&vu_vals[ch]);
       for(i = start_i; i<end ; i++){
         sample_t val=curr_in[i];
         data[pos++]=val * 32767.9; // weird lame format (lame is currently the only one using non-interleaved buffers, so the multiplication can be done here)
@@ -1580,7 +1606,7 @@ static void process_fill_buffer(sample_t *in[],buffer_t *buffer,int i,int end){
         if(val>max_vu)
           max_vu=val;
       }
-      vu_vals[ch] = max_vu;
+      safe_float_write(&vu_vals[ch], max_vu);
     }
   }
   //fprintf(stderr,"pos: %d %d\n",pos,num_channels);
@@ -1607,6 +1633,8 @@ static void process_fill_buffers(int jack_block_size){
   sample_t *in[num_channels];
   int i=0,ch;
 
+  jack_port_t **ports = ATOMIC_GET(g_ports);
+  
   for(ch=0;ch<num_channels;ch++)
     in[ch]=jack_port_get_buffer(ports[ch],jack_block_size);
       
@@ -1644,6 +1672,8 @@ enum{
 static int process_state=NOT_STARTED;
 
 static int process(jack_nframes_t nframes, void *arg){
+  (void)arg;
+
   jack_transport_state_t state=0;
 
   if(use_jack_transport==true){
@@ -1663,52 +1693,62 @@ static int process(jack_nframes_t nframes, void *arg){
       return 0;
   }
 
-  if(is_initialized==0)
+  if(ATOMIC_GET(is_initialized)==false)
     return 0;
 
-  if(is_running==0)
+  if(ATOMIC_GET(is_running)==false)
     return 0;
 
   if(process_state==RECORDING_FINISHED)
     return 0;
 
-	if (store_sync==0) {
+  jack_port_t **ports = ATOMIC_GET(g_ports);
+  
+  if (ATOMIC_GET(g_store_sync)==0) {
 #ifndef NEW_JACK_LATENCY_API
-		int ch;
-		j_latency=0;
-		for(ch=0;ch<num_channels;ch++) {
-			const jack_nframes_t jpl= jack_port_get_total_latency(client,ports[ch]);
-			if (j_latency < jpl) j_latency = jpl;
-		}
+    int ch;
+    j_latency=0;
+    for(ch=0;ch<num_channels;ch++) {
+      const jack_nframes_t jpl= jack_port_get_total_latency(client,ports[ch]);
+      if (j_latency < jpl) j_latency = jpl;
+    }
 #endif
-		clock_gettime(CLOCK_REALTIME, &rtime);
-		store_sync=1;
-	}
+    clock_gettime(CLOCK_REALTIME, &rtime);
+    ATOMIC_SET(g_store_sync, 1);
+  }
 
   if(fixed_duration==true){     // User has specified a duration
     int num_frames;
     
-    num_frames=JC_MIN(nframes,num_frames_to_record - num_frames_recorded);
+    num_frames=JC_MIN(nframes,num_frames_to_record - ATOMIC_GET(num_frames_recorded));
 
     if(num_frames>0)
       process_fill_buffers(num_frames);
 
-    num_frames_recorded += num_frames;
+    ATOMIC_ADD(num_frames_recorded, num_frames);
 
-    if(num_frames_recorded==num_frames_to_record){
+    if(ATOMIC_GET(num_frames_recorded)==num_frames_to_record){
       send_buffer_to_disk_thread(current_buffer);
+#ifdef __APPLE__
+      semaphore_signal(stop_sem);
+#else
       sem_post(&stop_sem);
+#endif
       process_state=RECORDING_FINISHED;
     }
 
   }else{
     process_fill_buffers(nframes);
-    num_frames_recorded += nframes;
+    ATOMIC_ADD(num_frames_recorded, nframes);
     if(    (use_jack_transport==true && state==JackTransportStopped)
 	|| (use_jack_freewheel==true && freewheel_mode==0)
       ){
       send_buffer_to_disk_thread(current_buffer);
+#ifdef __APPLE__
+      semaphore_signal(stop_sem);
+#else
       sem_post(&stop_sem);
+#endif
       process_state=RECORDING_FINISHED;
     }
   }
@@ -1719,8 +1759,9 @@ static int process(jack_nframes_t nframes, void *arg){
 }
 
 static int xrun(void *arg){
-	total_xruns++;
-	return 0;
+  (void)arg;
+  total_xruns++;
+  return 0;
 }
 
 
@@ -1844,6 +1885,8 @@ static int compare(const void *a, const void *b){
 static int reconnect_ports_questionmark(void){
   int ch;
 
+  jack_port_t **ports = ATOMIC_GET(g_ports);
+  
   for(ch=0;ch<num_channels;ch++){
     bool using_calloc;
     const char **connections1 = portnames_get_connections(ch, &using_calloc);
@@ -1926,21 +1969,35 @@ static void connect_ports(jack_port_t** ports){
   }
 }
 
+#ifdef __APPLE__
+static semaphore_t connection_semaphore;
+#else
 static sem_t connection_semaphore;
+#endif
+
 
 static void* connection_thread(void *arg){
+  (void)arg;
+
   while(1){
+#ifdef __APPLE__
+    semaphore_wait(connection_semaphore);
+#else
     sem_wait(&connection_semaphore);
-    if(is_running==0)
+#endif
+    if(ATOMIC_GET(is_running)==false)
       goto done;
     if(connect_meterbridge==1){
       connect_ports(ports_meterbridge);
       connect_meterbridge=0;
       continue;
     }
-    if(is_initialized && reconnect_ports_questionmark()){
+    if(ATOMIC_GET(is_initialized) && reconnect_ports_questionmark()){
       if(silent==false)
 	print_message("Reconnecting ports.\n");
+
+      jack_port_t **ports = ATOMIC_GET(g_ports);
+        
       disconnect_ports(ports);
       connect_ports(ports);
 
@@ -1957,11 +2014,19 @@ static void* connection_thread(void *arg){
 
 static void wake_up_connection_thread(void){
   if(use_manual_connections==false)
+#ifdef __APPLE__
+    semaphore_signal(connection_semaphore);
+#else
     sem_post(&connection_semaphore);
+#endif
 }
 
 static void start_connection_thread(void){
+#ifdef __APPLE__
+  semaphore_create(mach_task_self(), &connection_semaphore, SYNC_POLICY_FIFO, 0);
+#else
   sem_init(&connection_semaphore,0,0);
+#endif
   pthread_create(&connect_thread,NULL,connection_thread,NULL);
 }
 
@@ -1971,12 +2036,16 @@ static void stop_connection_thread(void){
 }
 
 static int graphordercallback(void *arg){
+  (void)arg;
+
   if (!freewheel_mode)
     wake_up_connection_thread();
   return 0;
 }
 
 static void freewheelcallback(int starting, void *arg){
+  (void)arg;
+
   freewheel_mode = starting;
 
   if (use_jack_freewheel==true && starting==0) {
@@ -1988,24 +2057,29 @@ static void freewheelcallback(int starting, void *arg){
 
 #if NEW_JACK_LATENCY_API
 static void jack_latency_cb(jack_latency_callback_mode_t mode, void *arg) {
-	int ch;
-	jack_latency_range_t jlty;
-	jack_nframes_t max_latency = 0;
+  (void)arg;
 
-	if (mode != JackCaptureLatency) return;
-	if (!ports) return;
+  int ch;
+  jack_latency_range_t jlty;
+  jack_nframes_t max_latency = 0;
+  
+  if (mode != JackCaptureLatency) return;
 
-	for(ch=0;ch<num_channels;ch++) {
-		if(ports[ch]==0) continue;
-		jack_port_get_latency_range(ports[ch], JackCaptureLatency, &jlty);
-		if (jlty.max > max_latency) max_latency= jlty.max;
-	}
-	j_latency = max_latency;
+  jack_port_t **ports = ATOMIC_GET(g_ports);
+  
+  if (!ports) return;
+  
+  for(ch=0;ch<num_channels;ch++) {
+    if(ports[ch]==0) continue;
+    jack_port_get_latency_range(ports[ch], JackCaptureLatency, &jlty);
+    if (jlty.max > max_latency) max_latency= jlty.max;
+  }
+  j_latency = max_latency;
 }
 #endif
 
 static void create_ports(void){
-  ports = (jack_port_t **) my_calloc (sizeof (jack_port_t *),num_channels);  
+  jack_port_t** ports = my_calloc (sizeof (jack_port_t *),num_channels);  
   {
     int ch;
     for(ch=0;ch<num_channels;ch++) {
@@ -2019,6 +2093,8 @@ static void create_ports(void){
       }
     }
   }
+
+  ATOMIC_SET(g_ports, ports);
 }
 
 
@@ -2032,14 +2108,24 @@ static void create_ports(void){
 
 
 static void finish(int sig){
+  (void)sig;
   //turn_on_echo(); //Don't think we can do this from a signal handler...
+#ifdef __APPLE__
+  semaphore_signal(stop_sem);
+#else
   sem_post(&stop_sem);
+#endif
 }
 
 static void jack_shutdown(void *arg){
+  (void)arg;
   fprintf(stderr,"jack_capture: JACK shutdown.\n");
   jack_has_been_shut_down=true;
+#ifdef __APPLE__
+  semaphore_signal(stop_sem);
+#else
   sem_post(&stop_sem);
+#endif
 }
 
 
@@ -2074,6 +2160,8 @@ static void start_jack(void){
 
 static pthread_t keypress_thread={0};
 static void* keypress_func(void* arg){
+  (void)arg;
+
   char gakk[64];
 
   turn_off_echo();
@@ -2094,7 +2182,11 @@ static void* keypress_func(void* arg){
 
   program_ended_with_return = true;
 
+#ifdef __APPLE__
+  semaphore_signal(stop_sem);
+#else
   sem_post(&stop_sem);
+#endif
   return NULL;
 }
 
@@ -2111,6 +2203,9 @@ static const char *advanced_help =
   "\"port\"     is by default set to the two first physical outputs. The \"port\" argument can be\n"
   "           specified more than once.\n"
   "\"filename\" is by default autogenerated to \"jack_capture_<number>.<format>\"\n"
+  "           Beware that you might risk overwriting an old file by using this option.\n"
+  "           By not using this option, you will not overwrite an old file.\n"
+  "           See also the option --filename-prefix\n"
   "\n"
   "\n"
   "Additional arguments:\n"
@@ -2163,7 +2258,9 @@ static const char *advanced_help =
   "                                    Default is 40. (Buffer is automatically increased during\n"
   "                                    recording when needed. But it will never go beyond this size.)\n"
   "[--filename] or [-fn]            -> Specify filename.\n"
-  "                                    (It's usually easier to set last argument instead)\n"
+  "                                    Beware that you might risk overwriting an old file by using this option.\n"
+  "                                    To only set prefix, use --filename-prefix / -fp instead.\n"
+  "                                    (It's usually easier to set last argument instead of using this option)\n"
   "[--osc] or [-O]                  -> Specify OSC port number to listen on. see --help-osc\n"
   "[--timestamp] or [-S]            -> create a FILENAME.tme file for each recording, storing\n"
   "                                    the system-time corresponding to the first audio sample.\n"
@@ -2253,7 +2350,7 @@ void init_arguments(int argc, char *argv[]){
                 "\n"
                 "To list advanced options, run\n"
                 "\n"
-                "     jack_capture --advanced-options\n"
+                "     jack_capture --advanced-options (or --help2)\n"
                 "\n"
                 )
     {
@@ -2482,6 +2579,12 @@ void init_various(void){
     buffers_init();
   }
 
+  if (access(base_filename, F_OK)==0){
+    print_message("\n");
+    print_message("!!! Warning, Overwriting existing file %s !!!\n", base_filename);
+    print_message("\n");
+  }
+
   verbose_print("main() Open soundfile and setup disk callback.\n");
   // Open soundfile and start disk thread
   {
@@ -2496,7 +2599,11 @@ void init_various(void){
   verbose_print("main() Init waiting.\n");
   // Init waiting.
   {
+#ifdef __APPLE__
+    semaphore_create(mach_task_self(), &stop_sem, SYNC_POLICY_FIFO, 0);
+#else
     sem_init(&stop_sem,0,0);
+#endif
     signal(SIGINT,finish);
     signal(SIGTERM,finish);
     if(no_stdin==false)
@@ -2527,7 +2634,7 @@ void init_various(void){
 
     create_ports();
     if(use_manual_connections==false)
-      connect_ports(ports);
+      connect_ports(ATOMIC_GET(g_ports));
   }
 
 
@@ -2535,7 +2642,7 @@ void init_various(void){
   // Everything initialized.
   //   (The threads are waiting for this variable, not the other way around, so now it just needs to be set.)
   {
-    is_initialized=1;
+    ATOMIC_SET(is_initialized, true);
     wake_up_connection_thread(); // Usually (?) not necessarry, but just in case.
   }
 
@@ -2587,8 +2694,15 @@ void wait_until_recording_finished(void){
   if(use_jack_freewheel==true)
     print_message("Waiting for Jack Freewheeling .\n");
   
+
+#ifdef __APPLE__
+  kern_return_t ret;
+  while((ret=semaphore_wait(stop_sem))!=KERN_SUCCESS)
+    print_message("Warning: semaphore_wait failed: %d",ret);
+#else
   while(sem_wait(&stop_sem)==-1)
     print_message("Warning: sem_wait failed: %s",strerror(errno));
+#endif
 
   turn_on_echo();
   if(helper_thread_running==1){
@@ -2607,7 +2721,7 @@ void wait_until_recording_finished(void){
 void stop_recording_and_cleanup(void){
   verbose_print("main() Stop recording and clean up.\n");
 
-  is_running=0;
+  ATOMIC_SET(is_running, false);
   
   if(use_manual_connections==false)
     stop_connection_thread();
@@ -2638,7 +2752,7 @@ void stop_recording_and_cleanup(void){
 }
 
 
-void append_argv(char **v1,char **v2,int len1,int len2,int max_size){
+void append_argv(char **v1,const char **v2,int len1,int len2,int max_size){
   int write_pos = len1;
   int read_pos  = 0;
 
@@ -2648,7 +2762,7 @@ void append_argv(char **v1,char **v2,int len1,int len2,int max_size){
   }
 
   while(write_pos<len1+len2)
-    v1[write_pos++] = v2[read_pos++];    
+    v1[write_pos++] = (char*)v2[read_pos++];    
 }
 
 
@@ -2677,7 +2791,7 @@ int main (int argc, char *argv[]){
   // get arguments both from command line and config file (config file is read first, so that command line can override)
   int c_argc;
   char **c_argv = read_config(&c_argc,500);
-  append_argv(c_argv,argv,c_argc,argc,500);
+  append_argv(c_argv,(const char**)argv,c_argc,argc,500);
   //print_argv(c_argv,c_argc+argc);
 
   init_arguments(c_argc+argc,c_argv);
@@ -2696,8 +2810,8 @@ int main (int argc, char *argv[]){
   stop_recording_and_cleanup();
 
   if (timemachine_mode==true && program_ended_with_return==true){
-    execv (org_argv[0], (char *const *) org_argv);
-    print_message("Error: exec returned.\n");
+    execvp (org_argv[0], (char *const *) org_argv);
+    print_message("Error: exec returned: %s.\n", strerror(errno));
     exit(127);
   }
 
