@@ -5,7 +5,11 @@
 
 #include <pthread.h>
 
+#ifdef __APPLE__
+#include <mach/mach.h>
+#else
 #include <semaphore.h>
+#endif
 
 #include "upwaker.h"
 
@@ -45,13 +49,21 @@ typedef struct vringbuffer_t{
   // Receiver callback
   pthread_t receiver_thread;
   upwaker_t *receiver_trigger;
+#ifdef __APPLE__
+  semaphore_t receiver_started;
+#else
   sem_t receiver_started;
+#endif
   Vringbuffer_receiver_callback receiver_callback;
 
   // Autoincrease callback
   pthread_t autoincrease_thread;
   upwaker_t *autoincrease_trigger;
+#ifdef __APPLE__
+  semaphore_t autoincrease_started;
+#else
   sem_t autoincrease_started;
+#endif
   Vringbuffer_autoincrease_callback autoincrease_callback;
   useconds_t autoincrease_interval;
 }vringbuffer_t;
